@@ -6,7 +6,7 @@ use vars qw/%Has/;
 BEGIN {
     $Has{diff}=!!eval "use Algorithm::Diff qw(sdiff diff); 1";
     $Has{sortkeys}=!!eval "Data::Dumper->new([1])->Sortkeys(1)->Dump()";
-}  
+}
 
 #$Id: test_helper.pl 26 2006-04-16 15:18:52Z demerphq $#
 
@@ -78,6 +78,30 @@ sub string_diff {
 
 sub capture { \@_ }
 
+sub _similar {
+    my ( $str1, $str2, $name, $obj ) = @_;
+
+    s/\s+$//gm for $str1,                          $str2;
+    s/\r\n/\n/g for $str1,                         $str2;
+    s/\(0x[0-9a-xA-X]+\)/(0xdeadbeef)/g for $str1, $str2;
+    my @vars = $str2 =~ m/^(?:my\s*)?(\$\w+)\s*=/gm;
+
+    #warn "@vars";
+    my $text = "\n" . $str1;
+    my $pat  = "\n" . $str2;
+
+    unless ( like( $text, $pat ) ) {
+        if ( $] >= 5.012 ) {
+            eval qq{
+                use re qw( Debug EXECUTE );
+                \$text =~ \$pat;
+                1;
+            }
+              or die $@;
+        }
+        $obj->diag;
+    }
+}
 sub _same {
     my ( $str1, $str2, $name, $obj ) = @_;
 
@@ -109,18 +133,19 @@ sub _same {
 
 sub _dumper {
     my ($todump)=@_;
-    my ($dump,$error);
+    my $dump;
+    my $error= "";
     foreach my $use_perl (1) {
         my $warned="";
         local $SIG{__WARN__}=sub { my $err=join ('',@_); $warned.=$err unless $err=~/^Subroutine|Encountered/};
         $dump=eval { scalar Data::Dumper->new( $todump )->Purity(1)->Sortkeys(1)->Quotekeys(1)->Useperl($use_perl)->Dump() };
-        unless ($@) {
+        if ( !$@ ) {
             normalize($dump);
-            return ($dump,$error.$warned);
-        }else {
+            return ($dump, $error . $warned);
+        } else {
             unless ($version) {
                 $version="\tSomething is wrong with Data::Dumper v" . Data::Dumper->VERSION . "\n";
-                $error=$version;
+                $error= $version;
             }
             my $msg=$@.$warned;
             unless ($errors{$msg}) {
@@ -159,7 +184,7 @@ sub normalize {
         s/\r\n/\n/g;
         s/\s+$//gm;
         $_.="\n";
-        
+
         #warn "<after>\n$_</after>\n";
     }
     unless (defined wantarray)  {
@@ -168,6 +193,69 @@ sub normalize {
     wantarray ? @x : $x[0]
 }
 
+sub similar {
+    goto &_similar unless ref( $_[1] );
+    my $name   = shift;
+    my $obj    = shift;
+    my ($expect,$result) = normalize(shift, scalar $obj->Data(@_)->Out());
+
+    my $main_pass = like( "\n$result", "\n$expect" );
+    if ( ! $main_pass ) {
+        $obj->diag;
+    }
+
+    my @declare=grep { /^[\$\@\%]/ } @{$obj->{declare}};
+
+    my @dump   =map  { /^[\@\%\&]/ ? "\\$_" : $_  } @{$obj->{out_names}};
+    my $dumpvars=join ( ",", @dump );
+
+    print $result,"\n" if $name=~/Test/;
+
+    my ($dumper,$error) = _dumper(\@_);
+    if ($error) {
+        diag( "$name\n$error" ) if $ENV{TEST_VERBOSE};
+    }
+    if ($dumper) {
+
+        my $result2_eval = $result . "\n" . 'scalar( $obj->Data(' . $dumpvars . ")->Out())\n";
+        my $dd_result_eval =
+          $result . "\nscalar(Data::Dumper->new("
+          . 'sub{\@_}->(' . $dumpvars . ")"
+          . ")->Purity(1)->Sortkeys(1)->Quotekeys(1)->"
+          . "Useperl(1)->Dump())\n";
+        unless ( $obj->Declare ) {
+            $dd_result_eval = "my(" . join ( ",", @declare ) . ");\n" . $dd_result_eval;
+            $result2_eval   = "my(" . join ( ",", @declare ) . ");\n" . $result2_eval;
+        }
+        foreach my $test ( [ "Data::Dumper", $dd_result_eval, $dumper ],
+                           [ "Data::Dump::Streamer", $result2_eval, $result ] ) {
+            my ( $test_name, $eval, $orig ) = @$test;
+
+            my ($warned,$res);
+            {
+                local $SIG{__WARN__}=sub { my $err=join ('',@_); $warned.=$err unless $err=~/^Subroutine|Encountered/};
+                $res  = eval $eval;
+                if ($warned) { print "Eval $test_name produced warnings:$warned\n$eval" };
+            }
+            normalize($res);
+            my $fail = 0;
+            if ($@) {
+                print join "\n", "Failed $test_name eval()", $eval, $@, "";
+                $fail = 1;
+            } elsif ( $res ne $orig ) {
+                print "Failed $test_name second time\n";
+                eval { print string_diff( $orig, $res, "Orig", "Result" ) };
+                print "Orig:\n$orig\nResult:\n$res\nEval:\n$eval\n";
+                $fail = 1;
+            }
+            $obj->diag if $fail;
+            return fail($name) if $fail;
+        }
+        #print join "\n",$result,$result2,$dumper,$dd_result,"";
+    }
+    ok( $main_pass, $name )
+}
+
 sub same {
     goto &_same unless ref( $_[1] );
     my $name   = shift;
@@ -324,7 +412,7 @@ my %mdchar=(u=>'|','+'=>'>','-'=>'<','c'=>'*');
 
 sub _my_diff {
     my ($e,$g,$mode)=@_;
-    
+
     unless ($Has{diff}) {
         if ($e ne $g) {
             return join "\n","Expected:",$e,"Got:",$g,""
@@ -332,7 +420,7 @@ sub _my_diff {
             return
         }
     }
-        
+
 
     my @exp=split /\n/,$e;
     my @got=split /\n/,$g;
@@ -419,7 +507,7 @@ sub test_dump {
 
     $test = {
                 name      => $test,
-          } 
+          }
         unless ref $test;
 
     $test->{pre_eval}= $::Pre_Eval unless exists $test->{pre_eval};
@@ -427,19 +515,19 @@ sub test_dump {
     $test->{no_dumper}= $::No_Dumper unless exists $test->{no_dumper};
     $test->{no_redump}= $::No_Redump unless exists $test->{no_redump};
 
-    $test->{verbose} = 1 
+    $test->{verbose} = 1
         if not exists $test->{verbose} and $ENV{TEST_VERBOSE};
 
     $test->{no_dumper} = 1 if !$Has{sortkeys};
 
     my @res=_dmp($obj,NO_EVAL,@_);
-    
+
     if (@res==2) {
         diag "Error:\n",$res[1];
         fail($test->{name});
         return
     }
-    
+
     my $to_dump=$obj->{out_names};
     my $to_decl=$obj->Declare ? [] : $obj->{declare}||[];
 
@@ -474,4 +562,4 @@ sub test_dump {
 
 
 
-1;
\ No newline at end of file
+1;
